/**********************************************************************
 * Node.js Website IP Scanner to capture visitor's IP via WebRTC.
 * Gathers and logs IP data, then cross-checks with multiple APIs 
 * for geolocation and VPN/proxy detection. Sends email and SMS 
 * notifications of visitor info, and stores logs for future analysis.
 * Written by Chad Jordan © interactivenexus.com
 * 2/14/2025
 *********************************************************************/

require('dotenv').config();
const express = require('express');
const axios = require('axios');
const fs = require('fs');
const nodemailer = require('nodemailer');
const twilio = require('twilio');

const app = express();
const port = process.env.PORT || 3000;

app.use(express.json());

//Route to log visitor IP
app.get('/track', async (req, res) => {
    const visitorIP = req.headers['x-forwarded-for'] || req.socket.remoteAddress;
    
    try {
        //Geolocation with IPinfo
        const geoResponse = await axios.get(`https://ipinfo.io/${visitorIP}?token=${process.env.IPINFO_TOKEN}`);
        const { ip, city, region, country, org } = geoResponse.data;

        //VPN/Proxy check with IPQualityScore
        const vpnResponse = await axios.get(`https://ipqualityscore.com/api/json/ip/${process.env.IPQUALITYSCORE_KEY}/${ip}`);
        const { proxy, vpn, tor, fraud_score } = vpnResponse.data;

        const vpnStatus = (proxy || vpn || tor) ? "Yes" : "No";
        const timestamp = new Date().toISOString();

        const logData = `IP: ${ip}, Location: ${city}, ${region}, ${country}, ISP: ${org}, VPN: ${vpnStatus}, Fraud Score: ${fraud_score}, Date: ${timestamp}\n`;

        //Save to log file
        fs.appendFileSync('visitors.log', logData);

        //Email Notification
        const transporter = nodemailer.createTransport({
            service: 'gmail',
            auth: {
                user: process.env.GMAIL_USER,
                pass: process.env.GMAIL_PASS
            }
        });

        const mailOptions = {
            from: process.env.GMAIL_USER,
            to: process.env.GMAIL_USER,
            subject: 'New Website Visitor',
            text: logData
        };

        await transporter.sendMail(mailOptions);

        //SMS Notification with Twilio
        const client = twilio(process.env.TWILIO_SID, process.env.TWILIO_AUTH_TOKEN);
        await client.messages.create({
            body: `New Visitor: ${ip}, Location: ${city}, ${region}, ${country}, VPN: ${vpnStatus}`,
            from: process.env.TWILIO_PHONE_NUMBER,
            to: process.env.YOUR_PHONE_NUMBER
        });

        res.send("Visitor logged.");
    } catch (error) {
        console.error(error);
        res.status(500).send("Error retrieving IP info.");
    }
});

app.listen(port, () => console.log(`Server running on port ${port}`));
